@@ -2,7 +2,6 @@ use core::{cmp::min, marker::PhantomData};
 use primitive_types::{H160, H256, U256};
 use sha3::{Digest, Keccak256};
 
-use super::precompiles::{PrecompileOutput, Precompiles};
 use crate::{
     collection::{vec::Vec, Map},
     db::Database,
@@ -14,9 +13,10 @@ use crate::{
     spec::{Spec, SpecId::*},
     subroutine::{Account, State, SubRoutine},
     util, CallContext, CreateScheme, ExitRevert, GlobalEnv, Inspector, Log, TransactOut,
-    TransactTo, Transfer, EVM,
+    TransactTo, Transfer, EVM, KECCAK_EMPTY,
 };
 use bytes::Bytes;
+use revm_precompiles::{Precompile, PrecompileOutput, Precompiles};
 
 pub struct EVMImpl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> {
     db: &'a mut DB,
@@ -37,15 +37,12 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVM for EVMImpl<'a, GSP
         gas_limit: u64,
         access_list: Vec<(H160, Vec<H256>)>,
     ) -> (ExitReason, TransactOut, u64, State) {
+        let exit_error = |reason: ExitReason| (reason, TransactOut::None, 0, State::new());
+
         if GSPEC::enabled(LONDON) {
             if let Some(priority_fee) = self.global_env.gas_priority_fee {
                 if priority_fee > self.global_env.gas_max_fee {
-                    return (
-                        ExitError::GasMaxFeeGreaterThanPriorityFee.into(),
-                        TransactOut::None,
-                        0,
-                        State::new(),
-                    );
+                    return exit_error(ExitError::GasMaxFeeGreaterThanPriorityFee.into());
                 }
             }
             let effective_gas_price = self.global_env.effective_gas_price();
@@ -55,63 +52,50 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVM for EVMImpl<'a, GSP
             // TODO maybe do this checks when creating evm. We already have all data there
             // or should be move effective_gas_price inside transact fn
             if effective_gas_price < basefee {
-                return (
-                    ExitError::GasPriceLessThenBasefee.into(),
-                    TransactOut::None,
-                    0,
-                    State::new(),
-                );
+                return exit_error(ExitError::GasPriceLessThenBasefee.into());
             }
             // check if priority fee is lower then max fee
         }
         // unusual to be found here, but check if gas_limit is more then block_gas_limit
         if U256::from(gas_limit) > self.global_env.block_gas_limit {
-            return (
-                ExitError::CallerGasLimitMoreThenBlock.into(),
-                TransactOut::None,
-                0,
-                State::new(),
-            );
+            return exit_error(ExitError::CallerGasLimitMoreThenBlock.into());
         }
 
         let mut gas = Gas::new(gas_limit);
-        //If there is no initial gas, should we take whatever is present?
+        // record initial gas cost.
         if !gas.record_cost(self.initialization::<GSPEC>(
             &data,
             matches!(transact_to, TransactTo::Create(_)),
             access_list,
         )) {
-            return (
-                ExitError::OutOfGas.into(),
-                TransactOut::None,
-                0,
-                State::new(),
-            );
+            return exit_error(ExitError::OutOfGas.into());
         }
 
         // load acc
         self.inner_load_account(caller);
 
+        // EIP-3607: Reject transactions from senders with deployed code
+        // This EIP is introduced after london but there was no colision in past
+        // so we can leave it enabled always
+        if self.subroutine.account(caller).info.code_hash != KECCAK_EMPTY {
+            return exit_error(ExitError::RejectCallerWithCode.into());
+        }
+
         // substract gas_limit*gas_price from current account.
-        let payment_value = U256::from(gas_limit) * self.global_env.effective_gas_price();
-        if !self.subroutine.balance_sub(caller, payment_value) {
-            return (
-                ExitError::LackOfFundForGasLimit.into(),
-                TransactOut::None,
-                0,
-                State::new(),
-            );
+        if let Some(payment_value) =
+            U256::from(gas_limit).checked_mul(self.global_env.effective_gas_price())
+        {
+            if !self.subroutine.balance_sub(caller, payment_value) {
+                return exit_error(ExitError::LackOfFundForGasLimit.into());
+            }
+        } else {
+            return exit_error(ExitError::OverflowPayment.into());
         }
 
         // check if we have enought balance for value transfer.
         let difference = self.global_env.gas_max_fee - self.global_env.effective_gas_price();
         if difference + value > self.subroutine.account(caller).info.balance {
-            return (
-                ExitError::OutOfFund.into(),
-                TransactOut::None,
-                0,
-                State::new(),
-            );
+            return exit_error(ExitError::OutOfFund.into());
         }
 
         // record all as cost;
@@ -163,7 +147,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         precompiles: Precompiles,
     ) -> Self {
         let mut precompile_acc = Map::new();
-        for add in precompiles.addresses() {
+        for (add, _) in precompiles.as_slice() {
             precompile_acc.insert(add.clone(), db.basic(add.clone()));
         }
         Self {
@@ -217,15 +201,15 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         is_create: bool,
         access_list: Vec<(H160, Vec<H256>)>,
     ) -> u64 {
-        for &ward_acc in self.precompiles.addresses().iter() {
+        for (ward_acc, _) in self.precompiles.as_slice() {
             //TODO trace load precompiles?
-            self.subroutine.load_account(ward_acc, self.db);
+            self.subroutine.load_account(ward_acc.clone(), self.db);
         }
 
         let zero_data_len = input.iter().filter(|v| **v == 0).count() as u64;
         let non_zero_data_len = (input.len() as u64 - zero_data_len) as u64;
         let (accessed_accounts, accessed_slots) = {
-            if SPEC::enabled(ISTANBUL) {
+            if SPEC::enabled(BERLIN) {
                 let mut accessed_slots = 0 as u64;
                 let accessed_accounts = access_list.len() as u64;
 
@@ -272,7 +256,6 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         init_code: Bytes,
         gas_limit: u64,
     ) -> (ExitReason, Option<H160>, Gas, Bytes) {
-        //println!("create depth:{}",self.subroutine.depth());
         let gas = Gas::new(gas_limit);
         self.load_account(caller);
 
@@ -302,10 +285,11 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         let checkpoint = self.subroutine.create_checkpoint();
 
         // create contract account and check for collision
-        if !self
-            .subroutine
-            .new_contract_acc(created_address, self.precompiles.addresses(), self.db)
-        {
+        if !self.subroutine.new_contract_acc(
+            created_address,
+            self.precompiles.contains(&created_address),
+            self.db,
+        ) {
             self.subroutine.checkpoint_revert(checkpoint);
             return (ExitError::CreateCollision.into(), ret, gas, Bytes::new());
         }
@@ -352,6 +336,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
                 } else {
                     // if we have enought gas
                     self.subroutine.checkpoint_commit();
+                    let code_hash = H256::from_slice(Keccak256::digest(&code).as_slice());
                     self.subroutine.set_code(created_address, code, code_hash);
                     (ExitSucceed::Returned.into(), ret, machine.gas, b)
                 }
@@ -409,11 +394,21 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
         }
 
         // call precompiles
-        if let Some(precompile) = self.precompiles.get_fun(&code_address) {
-            match (precompile)(input.as_ref(), gas_limit, &context, SPEC::IS_STATIC_CALL) {
+        if let Some(precompile) = self.precompiles.get(&code_address) {
+            let out = match precompile {
+                Precompile::Standard(fun) => fun(input.as_ref(), gas_limit),
+                Precompile::Custom(fun) => fun(input.as_ref(), gas_limit),
+            };
+            match out {
                 Ok(PrecompileOutput { output, cost, logs }) => {
                     if gas.record_cost(cost) {
-                        logs.into_iter().for_each(|l| self.subroutine.log(l));
+                        logs.into_iter().for_each(|l| {
+                            self.subroutine.log(Log {
+                                address: l.address,
+                                topics: l.topics,
+                                data: l.data,
+                            })
+                        });
                         self.subroutine.checkpoint_commit();
                         (ExitSucceed::Returned.into(), gas, Bytes::from(output))
                     } else {
@@ -423,7 +418,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> EVMImpl<'a, GSPEC, DB,
                 }
                 Err(e) => {
                     self.subroutine.checkpoint_revert(checkpoint); //TODO check if we are discarding or reverting
-                    (ExitReason::Error(e), gas, Bytes::new())
+                    (ExitError::Precompile(e).into(), gas, Bytes::new())
                 }
             }
         } else {
@@ -478,7 +473,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
         if INSPECT && is_cold {
             self.inspector.load_account(&address);
         }
-        (acc.info.code.clone().unwrap_or_default(), is_cold)
+        (acc.info.code.clone().unwrap(), is_cold)
     }
 
     /// Get code hash of address.
@@ -492,9 +487,7 @@ impl<'a, GSPEC: Spec, DB: Database, const INSPECT: bool> Handler
         }
 
         (
-            H256::from_slice(
-                Keccak256::digest(&acc.info.code.clone().unwrap_or_default()).as_slice(),
-            ),
+            H256::from_slice(Keccak256::digest(&acc.info.code.clone().unwrap()).as_slice()),
             is_cold,
         )
     }
